library IEEE;

entity translate_16e is
    Port ( 
			Enable_16		:IN	BIT;
			Instruction_IN	:IN	BIT_VECTOR (31 downto 0);
			Instruction_OUT:OUT	BIT_VECTOR (31 downto 0)
          );
end translate_16e;

architecture mips_16e of translate_16e is

signal tmp : BIT_VECTOR (31 downto 0);

---------------------------------------------------------------------
function translateRegister(Register_IN:BIT_VECTOR) return BIT_VECTOR is
	
	variable outRegister: BIT_VECTOR(4 downto 0) := (others => '0');
	
	begin
	
		outRegister(4 downto 3) := "00";
		outRegister(2 downto 0) := Register_IN;	
	
	return outRegister;

end translateRegister;
-----------------------------------------------------------------------
function translateInstruct(Instruct:BIT_VECTOR) return BIT_VECTOR is

	variable outValue: BIT_VECTOR(31 downto 0) := (others => '0');
	
	begin
	
		case Instruct(15 downto 11) is
		
			--RRR instruction (ADD-SUB)
			when "11100" =>
			
				outValue(31 downto 26) := "000000";					
				
				outValue(25 downto 21) := translateRegister(Instruct(10 downto 8));
				outValue(20 downto 16) := translateRegister(Instruct(7 downto 5));
				outValue(15 downto 11) := translateRegister(Instruct(4 downto 2));				
				
				outValue(5) := '1';
				outValue(1) := Instruct(1); --ADD / SUB
				
			--RR instruction (AND-OR)
			when "11101" =>
			
				outValue(31 downto 26) := "000000";					
				
				outValue(25 downto 21) := translateRegister(Instruct(10 downto 8));
				outValue(20 downto 16) := translateRegister(Instruct(7 downto 5));
				outValue(15 downto 11) := translateRegister(Instruct(10 downto 8));
				
				outValue(10 downto 1) := "0000010010";
				
				outValue(0) := Instruct(0); --AND / OR
				
			--LW
			when "10011" =>
				
				outValue(31 downto 26) := "100011";
				
				outValue(25 downto 21) := translateRegister(Instruct(10 downto 8));
				outValue(20 downto 16) := translateRegister(Instruct(7 downto 5));
				
				outValue(4 downto 0) := Instruct(4 downto 0);
				
			--SW
			when "11011" =>
				
				outValue(31 downto 26) := "101011";
				
				outValue(25 downto 21) := translateRegister(Instruct(10 downto 8));
				outValue(20 downto 16) := translateRegister(Instruct(7 downto 5));
				
				outValue(4 downto 0) := Instruct(4 downto 0);
				
			--J
			when "00010" =>
				
				outValue(31 downto 26) := "000010";				
				
				outValue(10 downto 0) := Instruct(10 downto 0);
				
			--BEQ
			when "00100" =>
			
				outValue(31 downto 26) := "000100";				
					
				outValue(25 downto 21) := translateRegister(Instruct(10 downto 8));
				
				outValue(7 downto 0) := Instruct(7 downto 0);
			
			when others =>
				outValue := (others => '0');
		
		end case;
	
	return outValue;

end translateInstruct;
------------------------------------------------------------------------

begin

	tmp <= Instruction_IN when Enable_16 = '0' else
		    translateInstruct(Instruction_IN);
	
	Instruction_OUT <= tmp;
		
end mips_16e;